
# Krb5 Client

    exports = module.exports = []

## Configuration

*   `krb5.kadmin_principal` (string, required)
*   `krb5.kadmin_password` (string, required)
*   `krb5.kadmin_server` (string, required)
*   `krb5.realm` (string, required)
*   `krb5.etc_krb5_conf` (object)
    Object representing the full ini file in "/etc/krb5.conf". It is
    generated by default.
*   `krb5.sshd` (object)
    Properties inserted in the "/etc/ssh/sshd_config" file.

Example:
```json
{
  "krb5": {
    "realm": "ADALTAS.COM",
    "kdc": "krb5.hadoop",
    "kadmin_server": "krb5.hadoop",
    "kadmin_principal": "wdavidw/admin@ADALTAS.COM",
    "kadmin_password": "test",
    "sshd": {
      "ChallengeResponseAuthentication: "yes",
      "KerberosAuthentication: "yes",
      "KerberosOrLocalPasswd: "yes",
      "KerberosTicketCleanup: "yes",
      "GSSAPIAuthentication: "yes",
      "GSSAPICleanupCredentials: "yes"
    }
  }
}
```

    module.exports.configure = (ctx) ->
      ctx.config.krb5 ?= {}
      ctx.config.krb5.sshd ?= {}
      ctx.config.krb5.kinit ?= '/usr/bin/kinit'
      etc_krb5_conf = misc.merge {}, module.exports.etc_krb5_conf, ctx.config.krb5.etc_krb5_conf
      ctx.config.krb5.etc_krb5_conf = etc_krb5_conf
      # Merge global with server-based configuration
      krb5_server_hosts = ctx.hosts_with_module "masson/core/krb5_server"
      for krb5_server_host in krb5_server_hosts
        {realms} = misc.merge {}, ctx.config.servers[krb5_server_host].krb5.etc_krb5_conf
        for realm, config of realms
          delete config.database_module
          realms[realm].kdc ?= krb5_server_host
          realms[realm].kdc = [realms[realm].kdc] unless Array.isArray realms[realm].kdc
          realms[realm].admin_server ?= krb5_server_host
          realms[realm].default_domain ?= realm.toLowerCase()
        etc_krb5_conf.realms = misc.merge {}, realms, etc_krb5_conf.realms
      # Generate the "domain_realm" property
      # for realm of etc_krb5_conf.realms
      #   etc_krb5_conf.domain_realm[".#{realm.toLowerCase()}"] ?= realm
      #   etc_krb5_conf.domain_realm["#{realm.toLowerCase()}"] ?= realm

    exports.push commands: 'install', modules: [
      'masson/core/krb5_server/wait'
      'masson/core/krb5_client/install'
      'masson/core/krb5_client/wait'
    ]

## Safe krb5 configuration

    module.exports.safe_etc_krb5_conf = safe_etc_krb5_conf = (etc_krb5_conf) ->
      etc_krb5_conf = krb5_server.safe_etc_krb5_conf etc_krb5_conf
      for realm, config of etc_krb5_conf.realms
        delete config.database_module
      delete etc_krb5_conf.dbmodules
      etc_krb5_conf

    module.exports.etc_krb5_conf =
      'logging':
        'default': 'SYSLOG:INFO:LOCAL1'
        'kdc': 'SYSLOG:NOTICE:LOCAL1'
        'admin_server': 'SYSLOG:WARNING:LOCAL1'
      'libdefaults':
        'dns_lookup_realm': false
        'dns_lookup_kdc': false
        'ticket_lifetime': '24h'
        'renew_lifetime': '7d'
        'forwardable': true
        'allow_weak_crypto': 'false'
        'ticket_lifetime': '24h'
        'clockskew': '300'
        'rdns': 'false'
      'realms': {}
      'domain_realm': {}
      'appdefaults':
        'pam':
          'debug': false
          'ticket_lifetime': 36000
          'renew_lifetime': 36000
          'forwardable': true
          'krb4_convert': false
      'dbmodules': {}

## Module Dependencies

    misc = require 'mecano/lib/misc'
    krb5_server = require '../krb5_server'
